import React, { Component, } from 'react';
import { Text, TouchableWithoutFeedback, View, StyleSheet, } from 'react-native';
import { isEqual } from '../Tools';
const NOOP = () => { };
const styles = StyleSheet.create({
    textItem: {
        flexDirection: 'row', justifyContent: 'space-between', alignItems: 'center',
    },
});
export class Props {
    constructor() {
        this.multiple = false;
        this.onChange = NOOP;
    }
}
export default class Select extends Component {
    constructor() {
        super(...arguments);
        this._values = [];
        this._selected = [];
        /**
         * 选中某一条(index)
         * @memberof Select
         */
        this.onSelect = (index) => {
            const { _values, _selected, props } = this;
            // 单选
            if (!props.multiple) {
                props.onChange(_values[index], index);
                return;
            }
            // 切换状态
            if (index in _selected) {
                delete _selected[index];
            }
            else {
                _selected[index] = _values[index];
            }
            // 执行回调
            props.onChange(_selected.filter(option => option !== undefined), Object.keys(_selected));
        };
        /**
         * 判断item是否被选中
         * @memberof Select
         */
        this.isSelected = (itemInstance, index) => {
            const { props } = this;
            const { props: itemProps } = itemInstance;
            if (!itemProps)
                return false;
            // 判断 selected 
            let isTrue = props.value === undefined && itemProps.selected;
            // 判断 value
            if (props.multiple) {
                isTrue = isTrue || props.value.findIndex(val => isEqual(val, itemProps.value)) !== -1;
            }
            else {
                if (this._selected.length > 1)
                    return false;
                isTrue = isTrue || isEqual(props.value, itemProps.value);
            }
            // 处理选中
            if (isTrue) {
                itemProps.selected[index] = props.value;
                return true;
            }
            return false;
        };
        /**
         * 渲染条子们
         * @memberof Select
         */
        this.renderItems = () => {
            const { props } = this;
            const arrayChildren = Array.isArray(props.children)
                ? props.children
                : props.children
                    ? [props.children]
                    : [];
            return arrayChildren.map((item, index) => {
                return this.renderItem(item, index);
            });
        };
        /**
         * 渲染单个item
         * @memberof Select
         */
        this.renderItem = (item, index) => {
            let itemInstance = item;
            if (itemInstance.type === Text && itemInstance.props.value === undefined) {
                itemInstance = React.createElement(Text, Object.assign({}, itemInstance.props, { value: itemInstance.props.children }));
            }
            this._values[index] = itemInstance.props.value;
            const isSelected = this.isSelected(itemInstance, index);
            // Text 兼容
            if (itemInstance.type === Text) {
                itemInstance = (React.createElement(View, { style: styles.textItem },
                    React.createElement(Text, Object.assign({}, itemInstance.props, { style: [{ backgroundColor: 'transparent' }, itemInstance.props.style] })),
                    isSelected ? React.createElement(Text, null, "\u221A") : null));
            }
            const onItemPress = () => this.onSelect(index);
            return (React.createElement(TouchableWithoutFeedback, { onPress: onItemPress, key: index },
                React.createElement(View, null,
                    React.createElement(itemInstance.type, Object.assign({}, itemInstance.props, { selected: isSelected })))));
        };
    }
    render() {
        const { style } = this.props;
        return (React.createElement(View, { style: style }, this.renderItems()));
    }
}
Select.defaultProps = new Props();
//# sourceMappingURL=index.js.map